@@ -141,18 +141,28 @@ class Fs_sip_profile_class extends Safe_configurable_object_class
     $this->set_field_category('user-agent-string','General');
   }
 
+  public function check_session_timeout($field_name)
+  {
+    $session_timeout = $this->get_data_value('session-timeout');
+    $minimum_session_expires = $this->get_data_value('minimum-session-expires');
+    if ($session_timeout < $minimum_session_expires) {
+      return array('check_session_timeout' => 'Session Expires must be equal or bigger than Minimum Session Expires');
+    }
+    return true;
+  }
+
   protected function configure_timing()
   {
     $this->add_enum_field('enable-timer', 'SIP Session Timer', 'dropdown', 'false', $this->enable_disable_enum());
     $this->set_field_help('enable-timer','Enable or Disable the SIP session timer.');
 
     $this->add_field('session-timeout', 'Session Expires', 'string', 1800);
-    $this->set_field_rules('session-timeout','required[timer]|is_numeric');
-    $this->set_field_help('session-timeout','Time in seconds for a specific session to expire.');
+    $this->set_field_rules('session-timeout','required[enable-timer]|is_numeric|greater_or_equal[90]|callback_check_session_timeout');
+    $this->set_field_help('session-timeout','Value in seconds for the Session-Expires header to be used in session expiration requests.');
 
-    $this->add_field('minimum-session-expires', 'Minimum Session Expires', 'string', 1800);
-    $this->set_field_rules('minimum-session-expires','required[timer]|is_numeric');
-    $this->set_field_help('minimum-session-expires','Minimum number of sessions that can be expired.');
+    $this->add_field('minimum-session-expires', 'Minimum Session Expires', 'string', 120);
+    $this->set_field_rules('minimum-session-expires','required[enable-timer]|is_numeric|greater_or_equal[90]');
+    $this->set_field_help('minimum-session-expires','Value in seconds for the Min-SE header indicating the minimum acceptable value in seconds for the Session-Expires header.');
 
     $this->add_field('rtcp-audio-interval-msec', 'RTCP Interval', 'string', '5000', 10);
     $this->set_field_rules('rtcp-audio-interval-msec', 'integer|greater_or_equal[100]|less_or_equal[5000]');
