@@ -34,7 +34,7 @@ function validate_account_filter($str)
     // 4/ 'regex=xxxxx' string
     $rc = (bool)FALSE;
     $str = trim($str);
-    if ('' == $str || 'unknown' == strtolower($str) || 'NULL' == $str || !strncmp($str, 'regex=', strlen('regex='))) {
+    if ('' == $str || 'unknown' == strtolower($str) || !strncmp($str, 'regex=', strlen('regex='))) {
         return (bool)TRUE;
     } else {
         // Coma separeted account list
@@ -101,7 +101,7 @@ class Nsc_sipsecmon_rules_config_class extends Safe_configurable_object_class
             "",
             "Below is the syntax that must be used for the source IP filter:",
             "",
-            "1) NULL = No source IP/Network filtering will be performed.",
+            "1) Empty = No source IP/Network filtering will be performed.",
             "2) IPv4 IP Address (10.10.0.1)",
             "3) IPv4 Network Address (172.16.0.0/16)",
             "4) Comma seperated list of IPv4 addresses or networks (10.10.0.1, 172.16.0.0/16)"
@@ -124,7 +124,7 @@ class Nsc_sipsecmon_rules_config_class extends Safe_configurable_object_class
             "",
             "Below is the syntax that must be used for the Account Registration filter:",
             "",
-            '1) NULL = No account filtering will be perfomed.',
+            '1) Empty = No account filtering will be perfomed.',
             '2) Unknown = The string "unknown" matches any accounts that are not known to the registrar (ie, trying to register joe@domain.com where the domain.com registrar does not know about any joe user). This option only works when SIP thru registration is disabled for the profile receiving the REGISTER request.',
             '3) Comma-separated list of accounts to match, ie "bob@company.com, alice@company.com".',
             '4) A regular expression matching the desired accounts in format: regex="the regular expression"',
@@ -136,7 +136,7 @@ class Nsc_sipsecmon_rules_config_class extends Safe_configurable_object_class
             "",
             "Below is the syntax that must be used for the user agent filter:",
             "",
-            '1) NULL = No user agent filtering will be perfomed.',
+            '1) Empty = No user agent filtering will be perfomed.',
             '2) Any string (this will be an exact match of the string with the user agent)',
             '3) A regular expression matching the desired user agent in format: regex="the regular expression"',
         ));
@@ -148,7 +148,7 @@ class Nsc_sipsecmon_rules_config_class extends Safe_configurable_object_class
         $this->set_field_help('action_expr', 'Action to perform when the rule matches');
         $this->add_field('action_param', 'Action Parameter', 'string', '', 50);
         // Set validation rule for block_ip as it's currently the only one supported
-        $this->set_field_rules('action_param', 'required[action_expr]|greater_or_equal[0]|less_than[1440]');
+        $this->set_field_rules('action_param', 'required[action_expr,in,block_ip]|greater_or_equal[0]|less_than[1440]');
         $this->set_field_help('action_param', array(
             "<b><u>Defines parameter of the selected action:</u></b>",
             "",
