@@ -23,6 +23,105 @@
 if (!defined('BASEPATH')) exit('No direct script access allowed');
 require_once ('application/helpers/safe_helper.php');
 safe_require_class('service');
+/*
+ * Some IP helper functions
+ * from  https://mebsd.com/coding-snipits/php-ipcalc-coding-subnets-ip-addresses.html
+ */
+
+// convert cidr to netmask
+// e.g. 21 = 255.255.248.0
+function cidr2netmask($cidr)
+{
+    for( $i = 1; $i <= 32; $i++ )
+        $bin .= $cidr >= $i ? '1' : '0';
+
+    $netmask = long2ip(bindec($bin));
+
+    if ( $netmask == "0.0.0.0")
+        return false;
+
+    return $netmask;
+}
+
+// get network address from cidr subnet
+// e.g. 10.0.2.56/21 = 10.0.0.0
+function cidr2network($ip, $cidr)
+{
+    $network = long2ip((ip2long($ip)) & ((-1 << (32 - (int)$cidr))));
+
+    return $network;
+}
+
+// convert netmask to cidr
+// e.g. 255.255.255.128 = 25
+function netmask2cidr($netmask)
+{
+    $bits = 0;
+    $netmask = explode(".", $netmask);
+
+    foreach($netmask as $octect)
+        $bits += strlen(str_replace("0", "", decbin($octect)));
+
+    return $bits;
+}
+
+// is ip in subnet
+// e.g. is 10.5.21.30 in 10.5.16.0/20 == true
+//      is 192.168.50.2 in 192.168.30.0/23 == false 
+function cidr_match($ip, $network, $cidr)
+{
+    if ((ip2long($ip) & ~((1 << (32 - $cidr)) - 1) ) == ip2long($network))
+    {
+        return true;
+    }
+
+    return false;
+}
+
+
+class Sng_network_route_add_config extends Safe_configuration_user_class 
+{
+    protected $_operator = 'add';
+
+    public function __construct($route_name, $data = array(), $action=Safe_configuration_class::CFG_CREATE)
+    {
+        parent::__construct($route_name, $action, $data);
+    }
+    public function run($node)
+    {
+        $data = $this->data();
+        $cidr = $data['prefix'];
+        $network = cidr2network($data['address'], $cidr);
+        // Prepare command:
+        //  ip route add <network-addr>/<network-mask-bits> [via <gateway-addr>] dev <ethernet-device-name>
+        $cmd = '/sbin/ip route '.$this->_operator.' ';
+        $cmd .= $network;
+        // Nework route or host route ?
+        if($cidr) {
+            $cmd .= '/'.$cidr;
+        }
+        if (strlen(trim($data['gateway']))){
+            $cmd .= ' via '.$data['gateway'];
+        }
+        $cmd .= ' dev '.$data['interface'];
+
+        $node->execute('sudo nohup ' . $cmd); 
+        // For now always return true
+        // Known valid usecase when ip route command fail:
+        //   * add a route on a down interface
+        //
+        return true;
+    }
+}
+
+class Sng_network_route_delete_config extends Sng_network_route_add_config
+{
+    protected $_operator = 'del';
+    public function __construct($route_name, $data = array(), $action=Safe_configuration_class::CFG_DELETE)
+    {
+        parent::__construct($route_name, $data, $action);
+    }
+}
 
 class Sng_network_route_class extends Safe_configurable_object_class
 {
@@ -30,6 +129,8 @@ class Sng_network_route_class extends Safe_configurable_object_class
     {
         // Parent constructor to invoke unserialize if needed
         parent::__construct($parent_name, $name, $node);
+        // Do NOT re-use records on update/delete
+        $this->_set_reuse(false);
     }
     public function configure()
     {
@@ -37,17 +138,34 @@ class Sng_network_route_class extends Safe_configurable_object_class
         $adapters = $this->_node->hardware()->adapters();
         $interfaces=array();
         foreach($adapters as $k => $v){
-            $interfaces[$k] = $k;
+            $if_ip = $v->ip_address();
+            $if_prefix = netmask2cidr($v->mask());
+            if($if_ip){
+                $label = $k . ' - ' . $if_ip . '/' . $if_prefix;
+                $interfaces[$k] = $label;
+            }
         }
+        $this->add_field('address', 'Destination', 'text', '',25);
+        $this->set_field_help('address', 'IP address.');
+        $this->set_field_rules('address', 'required|valid_ip');
+
+        $this->add_field('prefix', 'Prefix', 'text', '',5);
+        $this->set_field_help('prefix', 
+            array(
+                'The network prefix for the destination IP address.',
+                '',
+                'Optional: If not specified, a host rule is created.',
+            )
+            );
+        $this->set_field_rules('prefix', 'greater_than[1]|less_or_equal[32]');
+        $this->composite_layout('address', array('prefix'));
+
         $this->add_enum_field('interface', 'Network Interface', 'dropdown', '', $interfaces);
-        $this->set_field_help('interface', 'The Network Interface used to reach network number '.$this->name());
+        $this->set_field_help('interface', 'The network interface used to reach destination IP address.' );
 
-        $this->add_field('netmask', 'Network Mask', 'text', '',25);
-        $this->set_field_help('netmask', 'The netmask for the network number '.$this->name());
-        $this->set_field_rules('netmask', 'required|valid_ip');
         $this->add_field('gateway', 'Gateway', 'text', '',25);
-        $this->set_field_help('gateway', 'The default gateway, or an IP address that can be used to reach '.$this->name());
-        $this->set_field_rules('gateway', 'required|valid_ip');
+        $this->set_field_help('gateway', 'The default gateway, or an IP address that can be used to reach the destination IP address.');
+        $this->set_field_rules('gateway', 'valid_ip');
 
         return parent::configure();
     }
@@ -55,7 +173,50 @@ class Sng_network_route_class extends Safe_configurable_object_class
     public function description()
     {
         $data = $this->get_data_values();
-        return $data['interface'] . ' - ' . $data['netmask'] . ' - ' . $data['gateway'];
+        return $data['interface'] . ' / ' . $data['prefix'] . ' - ' . $data['gateway'];
+    }
+    public function summary($type , $long, $min)
+    {
+        return parent::summary($type, $long, 4);
+    }
+
+    public function validate($data, $output)
+    {
+        // Run regular validation first
+        $validate = parent::validate($data, $output);
+        if (!$validate) {
+            return $validate;
+        }
+        // Now take care of extra rules:
+        // 1/ destination must be unique
+        // 2/ if a gateway is specified, it must be on same subnet as interface
+        $network_module = $this->node()->find_module_by_name('network');
+        $routes = $network_module->api_retrieve_route();
+        foreach($routes as $route){
+            // Skip ourself
+            if($this->name() == $route->name()){
+                continue;
+            }
+            $route_data = $route->get_data_values(false);
+            // 1/ check unique route
+            if($data['address'] == $route_data['address']){
+                $validate = false;
+                $this->set_form_error('address', 'Destination already used by route '.$route->name());
+            }
+        }
+        // 2/ check gateway
+        if($data['gateway']){
+            // Get adapter and associated IP address
+            $if = $this->node()->hardware()->api_retrieve_adapter($data['interface']);
+            $if_ip = $if->ip_address();
+            $if_prefix = netmask2cidr($if->mask());
+            $validate = cidr_match($data['gateway'], cidr2network($if_ip, $if_prefix), $if_prefix);
+            if(!$validate){
+                $this->set_form_error('gateway', 'Gateway is not on '.$data['interface'].' subnet ('.$if_ip.'/'.$if_prefix.')');
+            }
+        }
+
+        return $validate;
     }
 }
 
@@ -82,16 +243,7 @@ class Sng_network_service_class extends Safe_service_class
             array(
                 'name' => 'Static Route',
                 'base_path' => $this->object_name() . '/route',
-                // Redmine #10005 - Remove dynamic reload from network routes
-                //'dynamic' => true,
-                'dynamic' => false,
-                /* Add some extra infos for config_manager create object */
-                'controller' => array( 
-                    'config_manager' => array(
-                        'new_label' => 'Destination IP address',
-                        'new_rule' => 'required|valid_ip',
-                    ),
-                ),
+                'dynamic' => true,
                 'methods' => array(
                     'create' => array(
                         'name' => 'Create',
@@ -146,9 +298,13 @@ class Sng_network_service_class extends Safe_service_class
      *
      * @return
      */
-    public function api_retrieve_route($name, $data=null, &$output = null) {
+    public function api_retrieve_route($name=null, $data=null, &$output = null) {
         $objs = $this->get_aggregate_objects('route');
-        return $objs[$name];
+        if($name){
+            return $objs[$name];
+        }else{
+            return $objs;
+        }
     }
     /**
      * @brief
@@ -160,13 +316,14 @@ class Sng_network_service_class extends Safe_service_class
      * @return
      */
     public function api_update_route($name, $data=null, &$output = null) {
+        // We cannot update static route object, must delete old one and create 
+        // a new one so serialization can occur correctly (ie. delete using ip 
+        // route del with EXACT same parameters as create.
+        // Only exception is if route is not yet serialized, then we can just 
+        // update as usual (ie. status == NEW)
         $route = $this->api_retrieve_route($name);
         if($route) {
-            if ($route->validate($data,$output)) {
-                if (true == $route->save()) {
-                    return true;
-                }
-            }
+            return $route->save();
         }
         return false;
     }
@@ -240,7 +397,7 @@ class Sng_network_service_class extends Safe_service_class
     public function post_write_config($obj_type=null)
     {
         // restart service
-        $this->restart();
+        //$this->restart();
         return parent::post_write_config($obj_type);
     }
     /**
@@ -252,6 +409,8 @@ class Sng_network_service_class extends Safe_service_class
      */
     private function _generate_config(&$config_manager, $obj_type=null)
     {
+        // Populate persistent routes
+        // 1st remove existing
         $adapters = $this->node()->hardware()->adapters();
         foreach($adapters as $if){
             $file = "/etc/sysconfig/network-scripts/route-".$if->name();
@@ -265,12 +424,18 @@ class Sng_network_service_class extends Safe_service_class
         // Loop around static routes to create interface route def
         $interfaces = array();
         foreach($this->get_aggregate_objects('route') as $route) {
-            $tmp = $route->get_data_values(false);
-            $if = $tmp['interface'];
-            unset($tmp['interface']);
-            $data['address'] = $route->name();
-            $data = array_merge($data,$tmp);
+            $data = $route->get_data_values(false);
+            $if = $data['interface'];
+            unset($data['interface']);
             $id = count($interfaces[$if]);
+            // For host route, need to add a /32 netmask
+            if(strlen(trim($data['prefix']))) {
+                $data['netmask'] = cidr2netmask($data['prefix']);
+                $data['address'] = cidr2network($data['address'], $data['prefix']);
+            }else{
+                $data['netmask'] = '255.255.255.255';
+            }
+            unset($data['prefix']);
 
             $route_def = array();
             foreach($data as $k => $v) {
@@ -295,14 +460,28 @@ class Sng_network_service_class extends Safe_service_class
                 new Safe_configuration_class(
                     $file, $content) );
         }
+        // we MUST handled deleted routes before new ones
+        // Loop around deleted static routes to create ip route del job
+        $obj_path = $this->aggregate_object_base_path('route');
+        $deleted_routes = Safe_object_serializer_class::get_serializer()->find_deleted_objects($obj_path);
+        foreach($deleted_routes as $route => $route_data) {
+            $config_manager->add_config(
+                new Sng_network_route_delete_config(
+                    $route, $route_data['data']) );
+        }
+
+        // Loop around static routes to create ip route add job
+        foreach($this->get_aggregate_objects('route') as $route) {
+            // Only on new route
+            if($route->status() == Safe_object_serializer_class::OBJ_STATUS_NEW) {
+                $route_data = $route->get_data_values(false);
+                $config_manager->add_config(
+                    new Sng_network_route_add_config(
+                        $route->name(), $route_data) );
+            }
+        }
 
         return true;
     }
-    public function new_fields(&$config_manager, $type)
-    {
-        $label = 'Destination IP address';
-        $config_manager->add_field('profile-name', $label, 'string', '', 50);
-        $config_manager->set_field_rules('profile-name', 'required|valid_ip');
-    }
 }
 /* End of file sng_firewall_service_class.php */
