@@ -144,7 +144,7 @@ class Sng_snmpd_config_class extends Safe_configurable_object_class
         //check system port
         foreach($ports as $port_item){
             if(!$this->node()->os()->valid_ip_port($port_item['ip'], $port_item['port'], $port_item['type'], 'snmpd')){
-                $error_msg_2 = " This port already using by other program now. " ;
+                $error_msg_2 = " This port is already being used by another program. " ;
             }
         }
         if($error_msg == '' && $error_msg_2 == ''){
@@ -466,7 +466,7 @@ class Sng_snmpd_service_class extends Safe_service_class
                 Safe_configuration_class::CFG_CHOWN));
         
         // snmpd.options.conf
-        $options_file = '/etc/sysconfig/snmpd.options';
+        $options_file = '/etc/sysconfig/snmpd';
         $snmpd_options_conf = array();
         $snmpd_options_conf[] = '# snmpd command line options';
         $listen = strtoupper($cfg_data['transport']).':';
