@@ -114,12 +114,18 @@ class Sng_transcoding_config_class extends Safe_configurable_object_class
         $media_int_group_field = array();
         $global_group_field = array();
         if ($internal_adapter) {
-            $this->add_field('enable_media_interface', 'enable_media_interface', 'hidden', 'true');
+            //$this->add_field('enable_media_interface', 'enable_media_interface', 'hidden', 'true');
             $default_udp_begin = '14000';
             $default_udp_end = '17999';
-        } else {
-            $this->add_enum_field('enable_media_interface', 'Enable/Disable Media Interfaces', 'dropdown', 'true', $this->enable_disable_enum());
-            $this->set_field_help('enable_media_interface', 'Option to enable/disable media interfaces from being used with the system.');
+        }else{
+            $default_udp_begin = '10000';
+            $default_udp_end = '20000';
+        }
+        $this->add_enum_field('enable_media_interface', 'Enable/Disable Media Interfaces', 'dropdown', 'true', $this->enable_disable_enum());
+        $this->set_field_help('enable_media_interface', 'Option to enable/disable media interfaces from being used with the system.');
+        if($internal_adapter){
+            $this->add_field('interfaces', 'interfaces', 'hidden', 'true');
+        }else{
             $this->add_multiple_field('interfaces', 'Detect Media Interfaces', 'inline_checkbox', $def_if, $if_name);
             $this->set_field_rules('interfaces', 'required[enable_media_interface]|required[,in,__empty__]');
             
@@ -129,10 +135,8 @@ class Sng_transcoding_config_class extends Safe_configurable_object_class
                 'All other media module information will be removed.'
             ));
             $media_int_group_field[] = 'interfaces[]';
-            $this->set_field_category('enable_media_interface', 'General');
-            $default_udp_begin = '10000';
-            $default_udp_end = '20000';
         }
+        $this->set_field_category('enable_media_interface', 'General');
         $this->add_enum_field('individual-ip', 'IP Mode', 'dropdown', 'false', $mode);
         // Set individual-ip help
         $this->set_field_help('individual-ip', '
@@ -143,8 +147,11 @@ class Sng_transcoding_config_class extends Safe_configurable_object_class
             You must connect an ethernet cable to the media ethernet port (D500/D100)
             ');
         $this->add_field('ip_address', 'Base IP address', 'text', '10.10.0.1');
-        if ($internal_adapter) $this->set_field_rules('ip_address', 'required[individual-ip]|valid_ip');
-        else $this->set_field_rules('ip_address', 'required|valid_ip');
+        if ($internal_adapter){
+            $this->set_field_rules('ip_address', 'required[individual-ip]|valid_ip');
+        }else{
+            $this->set_field_rules('ip_address', 'required|valid_ip');
+        }
         $this->set_field_help('ip_address', array(
             'Starting IP Address to be assigned to the media modules.',
             'For every module discovered, the IP address will be incremented by 1.'
@@ -310,6 +317,17 @@ class Sng_transcoding_config_class extends Safe_configurable_object_class
         $stop = $data['udp_end'];
     }
     public function validate($data, &$output){
+        $if_names = $this->detect_interfaces();
+        $internal_adapter = false;
+        $internal_adapter_list = array();
+        foreach ($if_names as $if => $value) {
+            if (Safe_hardware_adapter_class::TRANSCODING_ADAPTER == $value['type']) {
+                //Found internal adpaters
+                $internal_adapter = true;
+                $internal_adapter_list[] = $value['name'];
+            }
+        }
+
         $param_data = $this->get_data();
         if($param_data['enable_media_interface']['type'] == 'hidden'){
             unset($data['interfaces']);
@@ -320,6 +338,10 @@ class Sng_transcoding_config_class extends Safe_configurable_object_class
                 $data['interfaces'] = '';
                 $_POST['interfaces'] = '';
             }
+            if($internal_adapter && $data['enable_media_interface'] == 'true'){
+                $data['interfaces'] = implode(',', $internal_adapter_list);
+                $_POST['interfaces'] = $data['interfaces'];
+            }
         }
         $validate_result = parent::validate($data, $output);
         $new_data = $this->get_data_values(false);
