@@ -246,13 +246,66 @@ class Sng_config_manager extends Safe_CI_Controller
      * @param string $sub_cfg
      *                        Service/Module Name
      */
+    public function datatable_fetch($module_name, $obj_type=null, $obj_name=null, $_type=null)
+    {
+        $data = array();
+        $data['async'] = json_decode(trim(file_get_contents('php://input')), true);
+
+        // Retrieve module to work on
+        $module = $this->_the_app->find_module_by_name($module_name);
+        $base_url = $module->name();
+        $_id =  $module->name();
+        // Get config object
+        $cfg_obj = $module->config();
+        // Prepare Description
+        $description = $module->description();
+        $_define = $module->get_aggregate_object_definition($obj_type);
+        // Check if we want to work on an object instead of module
+        if($obj_type && $obj_name){
+            // Locate object
+            $_obj = $module->get_aggregate_object($obj_type, $obj_name);
+            if($_obj){
+                $description = $module->aggregate_object_name($obj_type).' - '.$obj_name;
+                $module = $_obj;
+                $cfg_obj = $module;
+            }
+            //$_type = $obj_type;
+            $_id .= '_'.$obj_type.'_'.$obj_name;
+        }else{
+            $_type = $obj_type;
+        }
+
+        // Check for custom stuff
+        $custom = array();
+        $custom = $_definition['controller']['config_manager'];
+        $table_heading_array = array();
+        $table = new Safe_crud_table_class();
+        $table->set_controller_url($this->router->fetch_class());
+        $data['table_id'] = $_id.'_'.$_type;
+        if($obj_type && $obj_name){
+            $base_url = $base_url . '/' . $obj_type.'/' . $obj_name;
+        }
+
+        $this->generate_table_data($table, $data, $module, $_type, $base_url, $custom, $table_heading_array);
+        unset($data['data']);
+        $_data = array();
+        $table->generate($_data);
+        foreach($_data as $v){
+            $row = array();
+            foreach($v as $v1){
+                $row[] = array_values($v1);
+            }
+            $data['data'][] = $row;
+        }
+
+        echo json_encode($data);
+    }
     public function modify($module_name, $obj_type=null, $obj_name=null)
     {
-        // verify if logged
-        //safe_authenticate ();
         if (!isset($module_name)) exit('Service/Module and product name required');
         // Retrieve module to work on
         $module = $this->_the_app->find_module_by_name($module_name);
+        $base_url = $module->name();
         // Get config object
         $cfg_obj = $module->config();
         // Prepare Description
@@ -274,16 +327,15 @@ class Sng_config_manager extends Safe_CI_Controller
         if ($action_buttons) {
             $data['notif_warn'][] = safe_form_global_action($description . ' actions ', $action_buttons);
         }
-
         $_id = $module->name();
 
         // Check if we want to work on an object instead of module
         if($obj_type && $obj_name){
             // Locate object
-            $aggregated_object = $module->get_aggregate_objects($obj_type);
-            if($aggregated_object[$obj_name]){
+            $_obj = $module->get_aggregate_object($obj_type, $obj_name);
+            if($_obj){
                 $description = $module->aggregate_object_name($obj_type).' - '.$obj_name;
-                $module = $aggregated_object[$obj_name];
+                $module = $_obj;
                 $cfg_obj = $module;
             }
             $_type = $obj_type;
@@ -304,7 +356,6 @@ class Sng_config_manager extends Safe_CI_Controller
         $this->load->view("safe_notifications", $data);
         //Show first table
         //if obj_type setted and obj name is null do not show module desc table
-       
         if($cfg_obj && !($obj_type != null && $obj_name == null) ) {
             $table = new Safe_crud_table_class();
             $data['table_size'] = '100%';
@@ -374,80 +425,13 @@ class Sng_config_manager extends Safe_CI_Controller
                 $data['table_title'] = $table->set_title($_definition['name']);
                 $data['table_id'] = $_id.'_'.$_type;
                 $table->set_controller_url($this->router->fetch_class());
-                
-                
-                $table_head = false;
-                $table_heading_array = array();
-                // Retrieve the objects
-                $aggregated_object = $module->get_aggregate_objects($_type);
-                if (!empty($aggregated_object)) {
-                    foreach ($aggregated_object as $object) {
-                        if (isset($object)) {
-                            // Check for singleton
-                            $has_sub = false;
-                            if($_definition['singleton']){
-                                $primary_key = $module_name. '/' . $_type;
-                                $fields = array($_definition['name']);
-                            }else{
-                                if($obj_type && $obj_name){
-                                    $primary_key = $module_name. '/' . $obj_type.'/' . $obj_name. '/' . $_type . '/' . $object->name();
-                                }else{
-                                    $primary_key = $module_name. '/' . $_type . '/' . $object->name();
-                                }
-                                $fields = array();
-                                $formatter = array();
-                                $summary_desc = $object->summary('horizontal', false);
-                                foreach($summary_desc['data'] as $key => $value ){
-                                    if(!$table_head){
-                                        //set table headings
-                                        $table_heading_array[] =  $key;
-                                    }
-                                    $fields[] = $value;
-                                }
-                                $has_sub = $object->has_aggregate_objects();
-                            }
-                            $table_head = true;
-                            
-                            // Button override ?
-                            if(isset($custom['summary_buttons'])){
-                                $line_button = $custom['summary_buttons'];
-                            }else{
-                                // Has sub object ?
-                                if($has_sub) {
-                                    if(!$_definition['readonly']){
-                                        $line_button =  'Modify|Delete:confirm';
-                                    }else{
-                                        $line_button = 'Modify[Details]';
-                                    }
-                                }else{
-                                    if(!$_definition['readonly']){
-                                        $line_button = 'Edit|Delete:confirm';
-                                    }else{
-                                        $line_button = 'Edit[Details]';
-                                    }
-                                }
-                                // Check if some methods to add as button
-                                foreach($_definition['methods'] as $_method => $method_def){
-                                    // Check if exists in controller class
-                                    $_method_name = $_method.'_'.$_type;
-                                    if(method_exists($this, $_method_name)){
-                                        $line_button .= '|'.$_method_name;
-                                        // with confirm ?
-                                        if(method_exists($this, $_method_name.'_confirm')){
-                                            $line_button .= ':confirm';
-                                        }
-                                    }
-                                }
-                            }
-                            // Add buttons
-                            $table_line = array(
-                                $table->add_row(
-                                $fields, 
-                                $primary_key, $line_button, '', true)
-                            );
-                        }
-                    }
+                if($obj_type && $obj_name){
+                    $base_url = $base_url . '/' . $obj_type.'/' . $obj_name;
                 }
+
+                $table_heading_array = array();
+                $this->generate_table_data($table, $data, $module, $_type, $base_url, $custom, $table_heading_array);
+
                 $table->set_headings($table_heading_array);
                 if(isset($custom['allow_create_button'])){
                     $allow_create_button = $custom['allow_create_button'];
@@ -487,8 +471,8 @@ class Sng_config_manager extends Safe_CI_Controller
                     // Yes, fetch table data and populate definition and object 
                     // data
                     $custom_data = array();
-                    $custom_data['table'] = $table->generate();
                     $custom_data['table_data'] = $table->generate_data();
+                    $custom_data['table'] = $table->generate();
                     $custom_data['url_base'] = base_url($this->router->fetch_class());
                     $custom_data['type'] = $_type;
                     $custom_data['definition'] = $_definition;
@@ -496,9 +480,11 @@ class Sng_config_manager extends Safe_CI_Controller
                     $custom_data['objects']['value'] = array();
                     $custom_data['objects']['name'] = array();
 
-                    $aggregated_object = $module->get_aggregate_objects($_type);
-                    if (!empty($aggregated_object)) {
-                        foreach ($aggregated_object as $object) {
+                    $obj_keys = $module->get_aggregate_objects_keys($_type);
+                    if (!empty($obj_keys)) {
+                        set_time_limit(0);
+                        foreach ($obj_keys as $key) {
+                            $object = $module->get_aggregate_object($_type, $key);
                             $custom_data['objects']['data'][] = $object->get_data();
                             $custom_data['objects']['value'][] = $object->get_data_values(false);
                             $custom_data['objects']['name'][] = $object->name();
@@ -510,6 +496,99 @@ class Sng_config_manager extends Safe_CI_Controller
         }
         $this->load->view("safe_footer", $data);
     }
+
+    protected function generate_table_data($table, &$data, $module, $_type, $base_url, $custom, &$table_heading_array)
+    {
+        $table_head = false;
+        $table_heading_array = array();
+        // Retrieve the objects keys
+        $obj_keys = $module->get_aggregate_objects_keys($_type);
+        // Do we have to asynch load the table ?
+        // Arbitrary value (100 :)
+        $max_count = 100;
+        if(count($obj_keys) > $max_count ) {
+            $total = count($obj_keys);
+            $offset = 0;
+            if($data['async']['offset']){
+                $offset = $data['async']['offset'];
+            }
+            $len = min($max_count, $total - $offset);
+            $obj_keys = array_splice($obj_keys, $offset, $len);
+
+            $data['async'] = array(
+                'url' => site_url($this->router->fetch_class().'/datatable_fetch/' . $base_url.'/'.$_type),
+                'id' => $data['table_id'],
+                'total' => $total,
+                'offset' => $offset,
+                'count' => $len,
+                );
+        }
+        if (!empty($obj_keys)) {
+            set_time_limit(0);
+            foreach ($obj_keys as $key) {
+                $object = $module->get_aggregate_object($_type, $key);
+                if (isset($object)) {
+                    // Check for singleton
+                    $has_sub = false;
+                    if($_definition['singleton']){
+                        $primary_key = $module->name(). '/' . $_type;
+                        $fields = array($_definition['name']);
+                    }else{
+                        $primary_key = $base_url . '/' . $_type . '/' . $object->name();
+                        $fields = array();
+                        $formatter = array();
+                        $summary_desc = $object->summary('horizontal', false);
+                        foreach($summary_desc['data'] as $key => $value ){
+                            if(!$table_head){
+                                //set table headings
+                                $table_heading_array[] =  $key;
+                            }
+                            $fields[] = $value;
+                        }
+                        $has_sub = $object->has_aggregate_objects();
+                    }
+                    $table_head = true;
+                    
+                    // Button override ?
+                    if(isset($custom['summary_buttons'])){
+                        $line_button = $custom['summary_buttons'];
+                    }else{
+                        // Has sub object ?
+                        if($has_sub) {
+                            if(!$_definition['readonly']){
+                                $line_button =  'Modify|Delete:confirm';
+                            }else{
+                                $line_button = 'Modify[Details]';
+                            }
+                        }else{
+                            if(!$_definition['readonly']){
+                                $line_button = 'Edit|Delete:confirm';
+                            }else{
+                                $line_button = 'Edit[Details]';
+                            }
+                        }
+                        // Check if some methods to add as button
+                        foreach($_definition['methods'] as $_method => $method_def){
+                            // Check if exists in controller class
+                            $_method_name = $_method.'_'.$_type;
+                            if(method_exists($this, $_method_name)){
+                                $line_button .= '|'.$_method_name;
+                                // with confirm ?
+                                if(method_exists($this, $_method_name.'_confirm')){
+                                    $line_button .= ':confirm';
+                                }
+                            }
+                        }
+                    }
+                    // Add buttons
+                    $table->add_row(
+                        $fields, 
+                        $primary_key, $line_button, '', true);
+
+                }
+            }
+        }
+    }
     /**
      * Function create a json encoded form to Create an aggregated object
      *   using popup dialog in browser
@@ -530,9 +609,9 @@ class Sng_config_manager extends Safe_CI_Controller
         $_type = $obj_type;
         // Add a sub object ?
         if($obj_name && $sub_type){
-            $aggregated_object = $module->get_aggregate_objects($obj_type);
-            if($aggregated_object[$obj_name]){
-                $module = $aggregated_object[$obj_name];
+            $_obj = $module->get_aggregate_object($obj_type, $obj_name);
+            if($_obj){
+                $module = $_obj;
                 $_type = $sub_type;
             }
         }
@@ -622,7 +701,7 @@ class Sng_config_manager extends Safe_CI_Controller
      * @return string $location
      * 							File location and filename of the server
      */
-    public function upload_file(&$name,&$content,&$msg,$seting=null)
+    public function upload_file(&$name,&$data,&$msg,$seting=null)
     {
         $status = "";
         $msg = "";
@@ -635,12 +714,13 @@ class Sng_config_manager extends Safe_CI_Controller
         $config = array();
     
         $config['upload_path'] = '/tmp/upload-files/';
-        if(isset($seting['file_type']) && $seting['file_type']=='csv'){
+        if(isset($seting['file_type'])){
             $config['allowed_types'] = $seting['file_type'];
         }else{
-            $config['allowed_types'] = 'pem';
+            $config['allowed_types'] = '*';
         }
-        $config['max_size']  = 1024 * 8;
+
+        $config['max_size']  = 1024 * 100;
         $config['encrypt_name'] = FALSE;
         $config['overwrite'] = TRUE;
     
@@ -658,27 +738,10 @@ class Sng_config_manager extends Safe_CI_Controller
         $status = "success";
         $msg = "File uploaded successfully.";
     
-        $name =  $_FILES['certificate']['name'];
         $data = $this->upload->data();
+        $name =  $data['raw_name'];
     
-        if(isset($seting['file_type']) && $seting['file_type']=='csv'){
-            ini_set('auto_detect_line_endings',TRUE);
-            $handle = fopen($data['full_path'],'r');
-            if ($handle) {
-                $csv_data = array();
-                while ( ($csv_data[] = fgetcsv($handle) ) !== FALSE ) {
-                }
-                fclose($handle);
-                ini_set('auto_detect_line_endings',FALSE);
-                $content = $csv_data;
-            }else{
-                @unlink($data['full_path']);
-                return false;
-            }
-        }else{
-        
-            $content = file_get_contents($data['full_path']);
-        }
+        $data['content'] = file_get_contents($data['full_path']);
     
         @unlink($data['full_path']);
         return true;
@@ -698,12 +761,13 @@ class Sng_config_manager extends Safe_CI_Controller
         // Get config object
         $cfg_obj = $module->config();
         $_type = $obj_type;
-        $aggregated_object = $module->get_aggregate_objects($obj_type);
+        //$aggregated_object = $module->get_aggregate_objects($obj_type);
 
         // Add a sub object ?
         if($obj_name && $sub_type){
-            if($aggregated_object[$obj_name]){
-                $module = $aggregated_object[$obj_name];
+            $_obj = $module->get_aggregate_object($obj_type, $obj_name);
+            if($_obj){
+                $module = $_obj;
                 $aggregated_object = $module->get_aggregate_objects($sub_type);
                 $_type = $sub_type;
             }
@@ -719,7 +783,7 @@ class Sng_config_manager extends Safe_CI_Controller
         $data_cfg = $module->get_aggregate_object_definition($_type);
         if (isset($data_cfg['autoname']) && $data_cfg['autoname'] == true){
             //generate name
-            $post['profile-name'] = Safe_object_serializer_class::build_unique_name($_type);
+            $post['profile-name'] = $module->build_unique_name($_type);
             $_POST['profile-name'] = $post['profile-name'];
         }
         $wizard_data = $param->get_data_values();
@@ -735,13 +799,15 @@ class Sng_config_manager extends Safe_CI_Controller
             //if a file uploaded
             if ($_FILES) {
                 $file_name = "";
-                $file_content = "";
                 $file_msg ='';
                 $file_data = array();
-                if($this->upload_file($file_name, $file_content, $file_msg)) {
-                    $file_data['profile-name'] = $file_name;
-                    $post['profile-name'] = $file_name;
-                    $file_data['base64_encode_contents'] = base64_encode($file_content);
+                $_data = array();
+                if($this->upload_file($file_name, $file_data, $file_msg, $data_cfg['file_def'])) {
+                    if(!isset($post['profile-name'])){
+                        $post['profile-name'] = $file_name;
+                    }
+                    $_data['base64_encode_contents'] = base64_encode($file_data['content']);
+                    $_data['file_name'] = $file_data['file_name'];
                 }
             }
             // Check we can create the object (delegate to module)
@@ -768,8 +834,32 @@ class Sng_config_manager extends Safe_CI_Controller
                 if ($new_obj) {
                     //if with a file uploaded
                     if ($_FILES) {
-                        if($new_obj->validate($file_data, $file_msg)){
+                        if($new_obj->validate($_data, $output)){
                             $post =  $post + $new_obj->get_data_values(false);
+                        } else {
+                            $data['message'] = "Fail to create <b>" . $post['profile-name'] . "</b> in " . $module->aggregate_object_name($_type);
+                            if($output != null){
+                                $data['message'] = $data['message'] . '<br>';
+                                foreach($output as $topic => $err){
+                                    $err_string = "";
+                                    if(is_array($err)){
+                                        foreach($err as $t2 => $r2){
+                                            if(is_array($r2)){
+                                                $err_string .= $t2 . ':' . implode(' ', array_values($r2)) . '<br/>';
+                                            }else{
+                                                $err_string .= $t2 . ':' . $r2 . '<br/>';
+                                            }
+                                        }
+                                    }else{
+                                        $err_string =  $topic .'=>'.$err;
+                                    }
+                                    $data['message'] = $data['message'] . '<br>'.$err_string;
+                                }
+                            }
+                            $data['message_type'] = 'alert';
+                            $data['result'] = false;
+                            echo json_encode($data);
+                            return;
                         }
                     }
                     if($obj_name && $sub_type){
@@ -822,15 +912,13 @@ class Sng_config_manager extends Safe_CI_Controller
         }
         // Retrieve module to work on
         $module = $this->_the_app->find_module_by_name($module_name);
-        $aggregated_object = $module->get_aggregate_objects($obj_type);
-        $aggr_object = $aggregated_object[$obj_name];
+        $aggr_object = $module->get_aggregate_object($obj_type, $obj_name);
         $description = $module->description();
         $_type = $obj_type;
         // Want to delete a child object ?
         if($sub_type && $sub_name){
             $module = $aggr_object;
-            $aggregated_object = $aggr_object->get_aggregate_objects($sub_type);
-            $aggr_object = $aggregated_object[$sub_name];
+            $aggr_object = $aggr_object->get_aggregate_object($sub_type, $sub_name);
             $_type = $sub_type;
             $description = $module->description();
         }
@@ -936,14 +1024,12 @@ class Sng_config_manager extends Safe_CI_Controller
         $del_obj_name = $obj_name;
         // Retrieve module to work on
         $module = $this->_the_app->find_module_by_name($module_name);
-        $aggregated_object = $module->get_aggregate_objects($obj_type);
-        $aggr_object = $aggregated_object[$obj_name];
+        $aggr_object = $module->get_aggregate_object($obj_type, $obj_name);
         $description = $module->description();
         // Want to delete a child object ?
         if($sub_type && $sub_name){
             $module = $aggr_object;
-            $aggregated_object = $aggr_object->get_aggregate_objects($sub_type);
-            $aggr_object = $aggregated_object[$sub_name];
+            $aggr_object = $aggr_object->get_aggregate_object($sub_type, $sub_name);
             $del_type = $sub_type;
             $del_obj_name = $sub_name;
         }
@@ -1104,8 +1190,7 @@ class Sng_config_manager extends Safe_CI_Controller
         
         //gets aggregated object based on the url. If editing aggregated object, set cfg_obj to the aggregated object configuration
         if ($obj_type && $obj_name) {
-            $aggr_obj = $module->get_aggregate_objects($obj_type);
-            $cfg_obj = $aggr_obj[$obj_name];
+            $cfg_obj = $module->get_aggregate_object($obj_type, $obj_name);
             $description = $module->aggregate_object_name($obj_type).' - '.$obj_name;
             if($cfg_obj) {
                 if($cfg_obj->has_aggregate_objects()){
@@ -1115,8 +1200,7 @@ class Sng_config_manager extends Safe_CI_Controller
                 // Same mechanism if editing a sub object
                 if($sub_type && $sub_name){
                     $description = $module->aggregate_object_name($sub_type).' - '.$sub_name;
-                    $aggr_obj = $module->get_aggregate_objects($sub_type);
-                    $cfg_obj = $aggr_obj[$sub_name];
+                    $cfg_obj = $module->get_aggregate_object($sub_type, $sub_name);
                 }
             }
         }elseif($obj_type){
@@ -1507,5 +1591,29 @@ class Sng_config_manager extends Safe_CI_Controller
             echo json_encode($data);
         }
     }
+    public function download($module_name, $obj_type, $obj_name=null, $sub_type=null, $sub_name=null)
+    {
+        // Retrieve module to work on
+        $module = $this->_the_app->find_module_by_name($module_name);
+        // Get config object
+        $cfg_obj = $module->config();
+        $_type = $obj_type;
+        $obj_keys = $module->get_aggregate_objects_keys($obj_type);
+        if($obj_name && $sub_type){
+            $_obj =  $module->get_aggregate_object($obj_type, $obj_name);
+            if($_obj){
+                $module = $_obj;
+                $_type = $sub_type;
+                $module = $module->get_aggregate_object($sub_type, $sub_name);
+            }
+        }
+        
+        if($module){
+            $module->download();
+        }else{
+            header('HTTP/1.0 404 Not Found');
+            exit();
+        }
+    }
 }
 /* End of file safe_edito.php */
